from base64 import b64decode
from algosdk.v2client.algod import AlgodClient
from algosdk.source_map import SourceMap


def get_algod_client(*, network):
    if network == "mainnet":
        algod_address = "https://mainnet-api.algonode.cloud"
    elif network == "testnet":
        algod_address = "https://testnet-api.algonode.cloud"
    elif network == "betanet":
        algod_address = "https://betanet-api.algonode.cloud"
    else:
        raise ValueError("Network is not valid.")

    algod_client = AlgodClient(
        algod_address=algod_address,
        algod_token=""
    )

    return algod_client


def get_teal(*, algod_client, application_id):
    approval_program = algod_client.application_info(application_id)['params']['approval-program']
    bytecode = b64decode(approval_program)
    teal = algod_client.algod_request("POST", '/teal/disassemble', data=bytecode)['result']
    return teal


def get_sourcemap(*, algod_client, teal):
    result = algod_client.compile(teal, source_map=True)
    sourcemap = SourceMap(result['sourcemap'])
    return sourcemap


def print_teal(*, teal, line_number, line_count=25):
    print(f"Line: {line_number}")

    teal_lines = teal.split('\n')
    for l in range(line_number - line_count, line_number + 1):
        print(l, teal_lines[l], '<---------- ' if l == line_number else '')


def inspect(*, network, application_id, program_counter, line_count):
    algod_client = get_algod_client(network=network)
    teal = get_teal(algod_client=algod_client, application_id=application_id)
    sourcemap = get_sourcemap(algod_client=algod_client, teal=teal)
    line_number = sourcemap.get_line_for_pc(program_counter)
    print_teal(teal=teal, line_number=line_number, line_count=line_count)



import argparse
from tealinspector.main import inspect


def run():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--network", type=str, help="mainnet, testnet or betanet. default is to mainnet.",
        const="mainnet", default="mainnet", nargs="?",
    )
    parser.add_argument("--application_id", type=int)
    parser.add_argument("--program_counter", type=int)
    parser.add_argument("--line_count", type=int, help="default is to 25.", const=25, default=25, nargs="?")

    arguments = parser.parse_args()

    inspect(
        network=arguments.network,
        application_id=arguments.application_id,
        program_counter=arguments.program_counter,
        line_count=arguments.line_count,
    )



import os
from setuptools import find_packages, setup


with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()


PACKAGE_NAME = "tealinspector"
VERSION = "0.1.0"
DESCRIPTION = "A teal inspector."
KEYWORDS = "teal algorand blockchain"
LICENSE = "MIT"
URL = "https://github.com/Hipo/tealinspector"
setup(
    name=PACKAGE_NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=README,
    long_description_content_type="text/markdown",
    url=URL,
    keywords=KEYWORDS,
    license=LICENSE,
    packages=find_packages(),
    include_package_data=True,
    package_data={"tealinspector": ["*.tx", "*.json"]},
    install_requires=["py-algorand-sdk >= 1.20"],
    python_requires=">=3.7",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "Intended Audience :: Information Technology",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    entry_points={
        "console_scripts": [
            "tealinspector = tealinspector.cli:run",
        ],
    },
)